#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import socket
import time
import ctypes
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')


class VideoControl():
    """
    <node>
        <interface name='com.kylin.mdm.InputControl.video'>
            <method name='DisableVideoDevices'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='EnableVideoDevices'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetVideoDevices'>
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'video-control','lv':'info','message':'{}'".format(
            time, name, ip, message)

        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")

    def DisableVideoDevices(self) -> int:
        try:
            self.log_transmit("开启摄像头设备管控")
        except Exception:
            print("log message error")

        if self.GetVideoDevices() == 0:
            return 0
        file = open("/etc/mdm/video", "w")
        file.write("0")
        file.close()
        self.so.disabledevices(ctypes.c_char_p(b"0e"))
        return 0

    def EnableVideoDevices(self) -> int:
        try:
            self.log_transmit("关闭摄像头设备管控")
        except Exception:
            print("log message error")

        if self.GetVideoDevices() == 1:
            return 0

        file = open("/etc/mdm/video", "w")
        file.write("1")
        file.close()

        try:
            blacklist_f = open("/etc/mdm/black-list", "r")
            blacklist = blacklist_f.readlines()
            blacklist_f.close()
        except Exception:
            blacklist = []

        try:
            uvclist_f = open("/etc/mdm/disable-video", "r")
            uvclist = set(uvclist_f.readlines())
            uvclist_f.close()
        except Exception:
            return 0

        for uvc in uvclist:
            if uvc not in blacklist:
                self.so.enabledevices(ctypes.c_char_p(bytes(uvc[0:4], encoding="utf8")),
                                      ctypes.c_char_p(bytes(uvc[4:8], encoding="utf8")))
        return 0

    def GetVideoDevices(self) -> int:
        try:
            file = open("/etc/mdm/video", "r")
            value = file.read(1)
            file.close()
        except Exception:
            return 1
        return int(value)


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.mdm.InputControl.video", VideoControl())
    loop.run()
